﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;

namespace FakeAsync
{
    public static class AsyncExtensions
    {
        #region System Extensions

        /// <summary>
        /// Asynchronously gets response from the internet using BeginGetResponse method.
        /// </summary>
        public static Async<WebResponse> GetResponseAsync(this WebRequest req)
        {
            return new AsyncPrimitive<WebResponse>(req.BeginGetResponse, req.EndGetResponse);
        }

        /// <summary>
        /// Asynchronously reads data from a stream using BeginRead.
        /// </summary>
        /// <param name="stream">The stream on which the method is called</param>
        /// <param name="buffer">The buffer to read the data into</param>
        /// <param name="offset">Byte offset in the buffer</param>
        /// <param name="count">Maximum number of bytes to read</param>
        /// <returns>Returns non-zero if there are still some data to read</returns>
        public static Async<int> ReadAsync(this Stream stream, byte[] buffer, int offset, int count)
        {
            return new AsyncPrimitive<int>(
                (callback, st) => stream.BeginRead(buffer, offset, count, callback, st),
                stream.EndRead);
        }

        /// <summary>
        /// Reads asynchronously the entire content of the stream and returns it 
        /// as a string using StreamReader.
        /// </summary>
        /// <returns>Returns string using the 'Result' class.</returns>
        public static IEnumerable<IAsync> ReadToEndAsync(this Stream stream)
        {
            MemoryStream ms = new MemoryStream();
            int read = -1;
            while (read != 0)
            {
                byte[] buffer = new byte[512];
                Async<int> count = stream.ReadAsync(buffer, 0, 512);
                yield return count;

                Console.WriteLine("[{0}] got data: {1}", "url", count.Result);
                ms.Write(buffer, 0, count.Result);
                read = count.Result;
            }

            //ms.Seek(0, SeekOrigin.Begin);
            //string s = new StreamReader(ms).ReadToEnd();

            //yield return new Result<string>(s);
        }

        #endregion

        
        public static void Execute(this IEnumerable<IAsync> async)
        {
            AsyncExtensions.Run(async.GetEnumerator());
        }

        

        
        internal static void Run(IEnumerator<IAsync> en)
        {
            if (!en.MoveNext()) return;
            en.Current.ExecuteStep
                (() => AsyncExtensions.Run(en));
        }

       
    }
}
